@@ -1,5 +1,6 @@
 from typing import Union
 from collections.abc import Mapping, MutableMapping
+from copy import deepcopy
 
 import numpy as np
 
@@ -126,11 +127,31 @@ class c3d(C3dMapper):
         else:
             self.c3d_swig = ezc3d.c3d(path)
 
+        self.extract_forceplat_data = extract_forceplat_data
         self._storage["header"] = c3d.Header(self.c3d_swig.header())
         self._storage["parameters"] = c3d.Parameter(self.c3d_swig.parameters())
-        self._storage["data"] = c3d.Data(self.c3d_swig, extract_forceplat_data)
+        self._storage["data"] = c3d.Data(self.c3d_swig, self.extract_forceplat_data)
         return
 
+    def __deepcopy__(self, memodict={}):
+        # Create a valid structure
+        new = c3d()
+        new.extract_forceplat_data = self.extract_forceplat_data
+        new._storage["header"] = c3d.Header(new.c3d_swig.header())
+        new._storage["parameters"] = c3d.Parameter(new.c3d_swig.parameters())
+        new._storage["data"] = c3d.Data(new.c3d_swig, new.extract_forceplat_data)
+
+        # Update the structure with a copy of all data
+        for header_key in self["header"]:
+            for value_key in self["header"][header_key]:
+                new["header"][header_key][value_key] = deepcopy(self["header"][header_key][value_key])
+        for group_key in self["parameters"]:
+            new["parameters"][group_key] = deepcopy(self["parameters"][group_key])
+        for data_key in self["data"]:
+            new["data"][data_key] = deepcopy(self["data"][data_key])
+
+        return new
+
     class Header(C3dMapper):
         def __init__(self, swig_header):
             super(c3d.Header, self).__init__()
@@ -567,22 +588,8 @@ class c3d(C3dMapper):
         for i in range(nb_analog_subframes):
             analogs.subframe(subframe)
 
-        # Fill the data
-        for f in range(nb_frames):
-            for i in range(nb_points):
-                pt.set(data_points[0, i, f], data_points[1, i, f], data_points[2, i, f])
-                pt.residual(data_meta_points["residuals"][0, i, f])
-                pt.cameraMask(data_meta_points["camera_masks"][:, i, f].tolist())
-                pts.point(pt, i)
-
-            for sf in range(nb_analog_subframes):
-                for i in range(nb_analogs):
-                    c.data(data_analogs[0, i, nb_analog_subframes * f + sf])
-                    subframe.channel(c, i)
-                analogs.subframe(subframe, sf)
-            frame = ezc3d.Frame()
-            frame.add(pts, analogs)
-            new_c3d.frame(frame)
+        # # Fill the data
+        new_c3d.import_numpy_data(data_points, data_meta_points["residuals"], data_meta_points["camera_masks"], data_analogs)
 
         # Write the file
         new_c3d.write(path)
